# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class VendorPurchase(BaseModel):
    """
    VendorPurchase
    """ # noqa: E501
    vendor_id: Optional[StrictStr] = Field(default=None, description="(v1.0) The vendorId property for the Dynamics 365 Business Central vendorPurchase entity", alias="vendorId")
    vendor_number: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="(v1.0) The vendorNumber property for the Dynamics 365 Business Central vendorPurchase entity", alias="vendorNumber")
    name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="(v1.0) The name property for the Dynamics 365 Business Central vendorPurchase entity")
    total_purchase_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="(v1.0) The totalPurchaseAmount property for the Dynamics 365 Business Central vendorPurchase entity", alias="totalPurchaseAmount")
    date_filter_filter_only: Optional[datetime] = Field(default=None, description="(v1.0) The dateFilter_FilterOnly property for the Dynamics 365 Business Central vendorPurchase entity", alias="dateFilter_FilterOnly")
    __properties: ClassVar[List[str]] = ["vendorId", "vendorNumber", "name", "totalPurchaseAmount", "dateFilter_FilterOnly"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of VendorPurchase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if total_purchase_amount (nullable) is None
        # and model_fields_set contains the field
        if self.total_purchase_amount is None and "total_purchase_amount" in self.model_fields_set:
            _dict['totalPurchaseAmount'] = None

        # set to None if date_filter_filter_only (nullable) is None
        # and model_fields_set contains the field
        if self.date_filter_filter_only is None and "date_filter_filter_only" in self.model_fields_set:
            _dict['dateFilter_FilterOnly'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of VendorPurchase from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "vendorId": obj.get("vendorId"),
            "vendorNumber": obj.get("vendorNumber"),
            "name": obj.get("name"),
            "totalPurchaseAmount": obj.get("totalPurchaseAmount"),
            "dateFilter_FilterOnly": obj.get("dateFilter_FilterOnly")
        })
        return _obj





# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr, field_validator
from typing import List, Optional
from typing_extensions import Annotated
from pybusinesscentral.model.list_sales_invoices200_response import ListSalesInvoices200Response
from pybusinesscentral.model.post_sales_invoice_request import PostSalesInvoiceRequest
from pybusinesscentral.model.sales_invoice import SalesInvoice

from pybusinesscentral.api_client import ApiClient, RequestSerialized
from pybusinesscentral.api_response import ApiResponse
from pybusinesscentral.rest import RESTResponseType


class SalesInvoiceApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def delete_sales_invoice(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Deletes an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_sales_invoice_with_http_info(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Deletes an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_sales_invoice_without_preload_content(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deletes an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_sales_invoice_serialize(
        self,
        company_id,
        sales_invoice_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if company_id is not None:
            _path_params['company_id'] = company_id
        if sales_invoice_id is not None:
            _path_params['salesInvoice_id'] = sales_invoice_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'oAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/companies({company_id})/salesInvoices({salesInvoice_id})',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_sales_invoice(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        select: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Selected properties to be retrieved")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SalesInvoice:
        """Retrieve the properties and relationships of an object of type salesInvoice for Dynamics 365 Business Central.


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param select: (v1.0) Selected properties to be retrieved
        :type select: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            expand=expand,
            select=select,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_sales_invoice_with_http_info(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        select: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Selected properties to be retrieved")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SalesInvoice]:
        """Retrieve the properties and relationships of an object of type salesInvoice for Dynamics 365 Business Central.


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param select: (v1.0) Selected properties to be retrieved
        :type select: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            expand=expand,
            select=select,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_sales_invoice_without_preload_content(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        select: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Selected properties to be retrieved")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve the properties and relationships of an object of type salesInvoice for Dynamics 365 Business Central.


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param select: (v1.0) Selected properties to be retrieved
        :type select: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            expand=expand,
            select=select,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_sales_invoice_serialize(
        self,
        company_id,
        sales_invoice_id,
        expand,
        select,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            '$expand': 'csv',
            '$select': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if company_id is not None:
            _path_params['company_id'] = company_id
        if sales_invoice_id is not None:
            _path_params['salesInvoice_id'] = sales_invoice_id
        # process the query parameters
        if expand is not None:
            
            _query_params.append(('$expand', expand))
            
        if select is not None:
            
            _query_params.append(('$select', select))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/companies({company_id})/salesInvoices({salesInvoice_id})',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_sales_invoices(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        top: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to return from the top of the list")] = None,
        skip: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to skip from the list")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to return from the list")] = None,
        filter: Annotated[Optional[StrictStr], Field(description="(v1.0) Filtering expression")] = None,
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        select: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Selected properties to be retrieved")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ListSalesInvoices200Response:
        """Returns a list of salesInvoices


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param top: (v1.0) Number of items to return from the top of the list
        :type top: int
        :param skip: (v1.0) Number of items to skip from the list
        :type skip: int
        :param limit: (v1.0) Number of items to return from the list
        :type limit: int
        :param filter: (v1.0) Filtering expression
        :type filter: str
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param select: (v1.0) Selected properties to be retrieved
        :type select: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_sales_invoices_serialize(
            company_id=company_id,
            top=top,
            skip=skip,
            limit=limit,
            filter=filter,
            expand=expand,
            select=select,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListSalesInvoices200Response",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_sales_invoices_with_http_info(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        top: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to return from the top of the list")] = None,
        skip: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to skip from the list")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to return from the list")] = None,
        filter: Annotated[Optional[StrictStr], Field(description="(v1.0) Filtering expression")] = None,
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        select: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Selected properties to be retrieved")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ListSalesInvoices200Response]:
        """Returns a list of salesInvoices


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param top: (v1.0) Number of items to return from the top of the list
        :type top: int
        :param skip: (v1.0) Number of items to skip from the list
        :type skip: int
        :param limit: (v1.0) Number of items to return from the list
        :type limit: int
        :param filter: (v1.0) Filtering expression
        :type filter: str
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param select: (v1.0) Selected properties to be retrieved
        :type select: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_sales_invoices_serialize(
            company_id=company_id,
            top=top,
            skip=skip,
            limit=limit,
            filter=filter,
            expand=expand,
            select=select,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListSalesInvoices200Response",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_sales_invoices_without_preload_content(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        top: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to return from the top of the list")] = None,
        skip: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to skip from the list")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="(v1.0) Number of items to return from the list")] = None,
        filter: Annotated[Optional[StrictStr], Field(description="(v1.0) Filtering expression")] = None,
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        select: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Selected properties to be retrieved")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Returns a list of salesInvoices


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param top: (v1.0) Number of items to return from the top of the list
        :type top: int
        :param skip: (v1.0) Number of items to skip from the list
        :type skip: int
        :param limit: (v1.0) Number of items to return from the list
        :type limit: int
        :param filter: (v1.0) Filtering expression
        :type filter: str
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param select: (v1.0) Selected properties to be retrieved
        :type select: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_sales_invoices_serialize(
            company_id=company_id,
            top=top,
            skip=skip,
            limit=limit,
            filter=filter,
            expand=expand,
            select=select,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListSalesInvoices200Response",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_sales_invoices_serialize(
        self,
        company_id,
        top,
        skip,
        limit,
        filter,
        expand,
        select,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            '$expand': 'csv',
            '$select': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if company_id is not None:
            _path_params['company_id'] = company_id
        # process the query parameters
        if top is not None:
            
            _query_params.append(('$top', top))
            
        if skip is not None:
            
            _query_params.append(('$skip', skip))
            
        if limit is not None:
            
            _query_params.append(('$limit', limit))
            
        if filter is not None:
            
            _query_params.append(('$filter', filter))
            
        if expand is not None:
            
            _query_params.append(('$expand', expand))
            
        if select is not None:
            
            _query_params.append(('$select', select))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/companies({company_id})/salesInvoices',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def patch_sales_invoice(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        content_type: Annotated[StrictStr, Field(description="(v1.0) application/json")],
        if_match: Annotated[StrictStr, Field(description="(v1.0) Required. When this request header is included and the eTag provided does not match the current tag on the entity, this will not be updated.")],
        post_sales_invoice_request: PostSalesInvoiceRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SalesInvoice:
        """Updates an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param content_type: (v1.0) application/json (required)
        :type content_type: str
        :param if_match: (v1.0) Required. When this request header is included and the eTag provided does not match the current tag on the entity, this will not be updated. (required)
        :type if_match: str
        :param post_sales_invoice_request: (required)
        :type post_sales_invoice_request: PostSalesInvoiceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            content_type=content_type,
            if_match=if_match,
            post_sales_invoice_request=post_sales_invoice_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def patch_sales_invoice_with_http_info(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        content_type: Annotated[StrictStr, Field(description="(v1.0) application/json")],
        if_match: Annotated[StrictStr, Field(description="(v1.0) Required. When this request header is included and the eTag provided does not match the current tag on the entity, this will not be updated.")],
        post_sales_invoice_request: PostSalesInvoiceRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SalesInvoice]:
        """Updates an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param content_type: (v1.0) application/json (required)
        :type content_type: str
        :param if_match: (v1.0) Required. When this request header is included and the eTag provided does not match the current tag on the entity, this will not be updated. (required)
        :type if_match: str
        :param post_sales_invoice_request: (required)
        :type post_sales_invoice_request: PostSalesInvoiceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            content_type=content_type,
            if_match=if_match,
            post_sales_invoice_request=post_sales_invoice_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def patch_sales_invoice_without_preload_content(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        content_type: Annotated[StrictStr, Field(description="(v1.0) application/json")],
        if_match: Annotated[StrictStr, Field(description="(v1.0) Required. When this request header is included and the eTag provided does not match the current tag on the entity, this will not be updated.")],
        post_sales_invoice_request: PostSalesInvoiceRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Updates an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param content_type: (v1.0) application/json (required)
        :type content_type: str
        :param if_match: (v1.0) Required. When this request header is included and the eTag provided does not match the current tag on the entity, this will not be updated. (required)
        :type if_match: str
        :param post_sales_invoice_request: (required)
        :type post_sales_invoice_request: PostSalesInvoiceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_sales_invoice_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            content_type=content_type,
            if_match=if_match,
            post_sales_invoice_request=post_sales_invoice_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _patch_sales_invoice_serialize(
        self,
        company_id,
        sales_invoice_id,
        content_type,
        if_match,
        post_sales_invoice_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if company_id is not None:
            _path_params['company_id'] = company_id
        if sales_invoice_id is not None:
            _path_params['salesInvoice_id'] = sales_invoice_id
        # process the query parameters
        # process the header parameters
        if content_type is not None:
            _header_params['Content-Type'] = content_type
        if if_match is not None:
            _header_params['If-Match'] = if_match
        # process the form parameters
        # process the body parameter
        if post_sales_invoice_request is not None:
            _body_params = post_sales_invoice_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/companies({company_id})/salesInvoices({salesInvoice_id})',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def post_action_sales_invoices(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Performs the post action for salesInvoices entity


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_action_sales_invoices_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_action_sales_invoices_with_http_info(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Performs the post action for salesInvoices entity


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_action_sales_invoices_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_action_sales_invoices_without_preload_content(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        sales_invoice_id: Annotated[StrictStr, Field(description="(v1.0) id for salesInvoice")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Performs the post action for salesInvoices entity


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param sales_invoice_id: (v1.0) id for salesInvoice (required)
        :type sales_invoice_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_action_sales_invoices_serialize(
            company_id=company_id,
            sales_invoice_id=sales_invoice_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _post_action_sales_invoices_serialize(
        self,
        company_id,
        sales_invoice_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if company_id is not None:
            _path_params['company_id'] = company_id
        if sales_invoice_id is not None:
            _path_params['salesInvoice_id'] = sales_invoice_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'oAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/companies({company_id})/salesInvoices({salesInvoice_id})/Microsoft.NAV.post',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def post_sales_invoice(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        content_type: Annotated[StrictStr, Field(description="(v1.0) application/json")],
        post_sales_invoice_request: PostSalesInvoiceRequest,
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SalesInvoice:
        """Creates an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param content_type: (v1.0) application/json (required)
        :type content_type: str
        :param post_sales_invoice_request: (required)
        :type post_sales_invoice_request: PostSalesInvoiceRequest
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_sales_invoice_serialize(
            company_id=company_id,
            content_type=content_type,
            post_sales_invoice_request=post_sales_invoice_request,
            expand=expand,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_sales_invoice_with_http_info(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        content_type: Annotated[StrictStr, Field(description="(v1.0) application/json")],
        post_sales_invoice_request: PostSalesInvoiceRequest,
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SalesInvoice]:
        """Creates an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param content_type: (v1.0) application/json (required)
        :type content_type: str
        :param post_sales_invoice_request: (required)
        :type post_sales_invoice_request: PostSalesInvoiceRequest
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_sales_invoice_serialize(
            company_id=company_id,
            content_type=content_type,
            post_sales_invoice_request=post_sales_invoice_request,
            expand=expand,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_sales_invoice_without_preload_content(
        self,
        company_id: Annotated[StrictStr, Field(description="(v1.0) id for company")],
        content_type: Annotated[StrictStr, Field(description="(v1.0) application/json")],
        post_sales_invoice_request: PostSalesInvoiceRequest,
        expand: Annotated[Optional[List[StrictStr]], Field(description="(v1.0) Entities to expand")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Creates an object of type salesInvoice in Dynamics 365 Business Central


        :param company_id: (v1.0) id for company (required)
        :type company_id: str
        :param content_type: (v1.0) application/json (required)
        :type content_type: str
        :param post_sales_invoice_request: (required)
        :type post_sales_invoice_request: PostSalesInvoiceRequest
        :param expand: (v1.0) Entities to expand
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_sales_invoice_serialize(
            company_id=company_id,
            content_type=content_type,
            post_sales_invoice_request=post_sales_invoice_request,
            expand=expand,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "SalesInvoice",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _post_sales_invoice_serialize(
        self,
        company_id,
        content_type,
        post_sales_invoice_request,
        expand,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            '$expand': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if company_id is not None:
            _path_params['company_id'] = company_id
        # process the query parameters
        if expand is not None:
            
            _query_params.append(('$expand', expand))
            
        # process the header parameters
        if content_type is not None:
            _header_params['Content-Type'] = content_type
        # process the form parameters
        # process the body parameter
        if post_sales_invoice_request is not None:
            _body_params = post_sales_invoice_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/companies({company_id})/salesInvoices',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

# flake8: noqa

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import ApiClient
from pybusinesscentral.api_response import ApiResponse
from pybusinesscentral.api_client import ApiClient
from pybusinesscentral.configuration import Configuration
from pybusinesscentral.exceptions import OpenApiException
from pybusinesscentral.exceptions import ApiTypeError
from pybusinesscentral.exceptions import ApiValueError
from pybusinesscentral.exceptions import ApiKeyError
from pybusinesscentral.exceptions import ApiAttributeError
from pybusinesscentral.exceptions import ApiException


__all__ = [
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
]


# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBytes, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated
from pybusinesscentral.model.general_ledger_entry import GeneralLedgerEntry
from typing import Optional, Set
from typing_extensions import Self

class GeneralLedgerEntryAttachments(BaseModel):
    """
    GeneralLedgerEntryAttachments
    """ # noqa: E501
    general_ledger_entry_number: Optional[StrictInt] = Field(default=None, description="(v1.0) The generalLedgerEntryNumber property for the Dynamics 365 Business Central generalLedgerEntryAttachments entity", alias="generalLedgerEntryNumber")
    id: Optional[StrictStr] = Field(default=None, description="(v1.0) The id property for the Dynamics 365 Business Central generalLedgerEntryAttachments entity")
    file_name: Optional[Annotated[str, Field(strict=True, max_length=250)]] = Field(default=None, description="(v1.0) The fileName property for the Dynamics 365 Business Central generalLedgerEntryAttachments entity", alias="fileName")
    byte_size: Optional[StrictInt] = Field(default=None, description="(v1.0) The byteSize property for the Dynamics 365 Business Central generalLedgerEntryAttachments entity", alias="byteSize")
    content: Optional[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]] = Field(default=None, description="(v1.0) The content property for the Dynamics 365 Business Central generalLedgerEntryAttachments entity")
    created_date_time: Optional[datetime] = Field(default=None, description="(v1.0) The createdDateTime property for the Dynamics 365 Business Central generalLedgerEntryAttachments entity", alias="createdDateTime")
    general_ledger_entry: Optional[GeneralLedgerEntry] = Field(default=None, alias="generalLedgerEntry")
    __properties: ClassVar[List[str]] = ["generalLedgerEntryNumber", "id", "fileName", "byteSize", "content", "createdDateTime", "generalLedgerEntry"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GeneralLedgerEntryAttachments from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of general_ledger_entry
        if self.general_ledger_entry:
            _dict['generalLedgerEntry'] = self.general_ledger_entry.to_dict()
        # set to None if file_name (nullable) is None
        # and model_fields_set contains the field
        if self.file_name is None and "file_name" in self.model_fields_set:
            _dict['fileName'] = None

        # set to None if byte_size (nullable) is None
        # and model_fields_set contains the field
        if self.byte_size is None and "byte_size" in self.model_fields_set:
            _dict['byteSize'] = None

        # set to None if content (nullable) is None
        # and model_fields_set contains the field
        if self.content is None and "content" in self.model_fields_set:
            _dict['content'] = None

        # set to None if created_date_time (nullable) is None
        # and model_fields_set contains the field
        if self.created_date_time is None and "created_date_time" in self.model_fields_set:
            _dict['createdDateTime'] = None

        # set to None if general_ledger_entry (nullable) is None
        # and model_fields_set contains the field
        if self.general_ledger_entry is None and "general_ledger_entry" in self.model_fields_set:
            _dict['generalLedgerEntry'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GeneralLedgerEntryAttachments from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "generalLedgerEntryNumber": obj.get("generalLedgerEntryNumber"),
            "id": obj.get("id"),
            "fileName": obj.get("fileName"),
            "byteSize": obj.get("byteSize"),
            "content": obj.get("content"),
            "createdDateTime": obj.get("createdDateTime"),
            "generalLedgerEntry": GeneralLedgerEntry.from_dict(obj["generalLedgerEntry"]) if obj.get("generalLedgerEntry") is not None else None
        })
        return _obj





# coding: utf-8

"""
(v2.0) Dynamics 365 Business Central

(v2.0) Business Central Standard APIs

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from setuptools import find_packages, setup  # noqa: H301

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools
NAME = "pybusinesscentral"
VERSION = "3.3.0"
PYTHON_REQUIRES = ">= 3.9"
REQUIRES = [
    "urllib3 >= 2.1.0, < 3.0.0",
    "python-dateutil >= 2.8.2",
    "pydantic >= 2",
    "typing-extensions >= 4.7.1",
]

setup(
    name=NAME,
    version=VERSION,
    description="(v2.0) Dynamics 365 Business Central",
    author="Uptick",
    author_email="support@uptickhq.com",
    url="",
    keywords=["OpenAPI", "OpenAPI-Generator", "(v2.0) Dynamics 365 Business Central"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description="""\
    (v2.0) Business Central Standard APIs
    """,  # noqa: E501
    package_data={"pybusinesscentral": ["py.typed"]},
)



# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from pybusinesscentral.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from pybusinesscentral.model.account import Account
from pybusinesscentral.model.aged_accounts_payable import AgedAccountsPayable
from pybusinesscentral.model.aged_accounts_receivable import AgedAccountsReceivable
from pybusinesscentral.model.attachments import Attachments
from pybusinesscentral.model.balance_sheet import BalanceSheet
from pybusinesscentral.model.bank_account import BankAccount
from pybusinesscentral.model.cash_flow_statement import CashFlowStatement
from pybusinesscentral.model.company import Company
from pybusinesscentral.model.company_information import CompanyInformation
from pybusinesscentral.model.country_region import CountryRegion
from pybusinesscentral.model.currency import Currency
from pybusinesscentral.model.customer import Customer
from pybusinesscentral.model.customer_financial_detail import CustomerFinancialDetail
from pybusinesscentral.model.customer_payment import CustomerPayment
from pybusinesscentral.model.customer_payment_journal import CustomerPaymentJournal
from pybusinesscentral.model.customer_sale import CustomerSale
from pybusinesscentral.model.default_dimensions import DefaultDimensions
from pybusinesscentral.model.dimension import Dimension
from pybusinesscentral.model.dimension_line import DimensionLine
from pybusinesscentral.model.dimension_set_line import DimensionSetLine
from pybusinesscentral.model.dimension_value import DimensionValue
from pybusinesscentral.model.dimensiontype import Dimensiontype
from pybusinesscentral.model.documentlineobjectdetailstype import Documentlineobjectdetailstype
from pybusinesscentral.model.employee import Employee
from pybusinesscentral.model.general_ledger_entry import GeneralLedgerEntry
from pybusinesscentral.model.general_ledger_entry_attachments import GeneralLedgerEntryAttachments
from pybusinesscentral.model.income_statement import IncomeStatement
from pybusinesscentral.model.item import Item
from pybusinesscentral.model.item_category import ItemCategory
from pybusinesscentral.model.itemunitofmeasureconversiontype import Itemunitofmeasureconversiontype
from pybusinesscentral.model.journal import Journal
from pybusinesscentral.model.journal_line import JournalLine
from pybusinesscentral.model.payment_method import PaymentMethod
from pybusinesscentral.model.payment_term import PaymentTerm
from pybusinesscentral.model.pdf_document import PdfDocument
from pybusinesscentral.model.picture import Picture
from pybusinesscentral.model.post_customer_request import PostCustomerRequest
from pybusinesscentral.model.post_dimension_set_line_request import PostDimensionSetLineRequest
from pybusinesscentral.model.post_purchase_invoice_request import PostPurchaseInvoiceRequest
from pybusinesscentral.model.post_purchase_invoice_line_for_purchase_invoice_request import PostPurchaseInvoiceLineForPurchaseInvoiceRequest
from pybusinesscentral.model.post_sales_credit_memo_request import PostSalesCreditMemoRequest
from pybusinesscentral.model.post_sales_credit_memo_line_for_sales_credit_memo_request import PostSalesCreditMemoLineForSalesCreditMemoRequest
from pybusinesscentral.model.post_sales_invoice_request import PostSalesInvoiceRequest
from pybusinesscentral.model.post_sales_invoice_line_for_sales_invoice_request import PostSalesInvoiceLineForSalesInvoiceRequest
from pybusinesscentral.model.post_vendor_request import PostVendorRequest
from pybusinesscentral.model.postaladdresstype import Postaladdresstype
from pybusinesscentral.model.project import Project
from pybusinesscentral.model.purchase_invoice import PurchaseInvoice
from pybusinesscentral.model.purchase_invoice_line import PurchaseInvoiceLine
from pybusinesscentral.model.retained_earnings_statement import RetainedEarningsStatement
from pybusinesscentral.model.sales_credit_memo import SalesCreditMemo
from pybusinesscentral.model.sales_credit_memo_line import SalesCreditMemoLine
from pybusinesscentral.model.sales_invoice import SalesInvoice
from pybusinesscentral.model.sales_invoice_line import SalesInvoiceLine
from pybusinesscentral.model.sales_credit_memo import SalesCreditMemo
from pybusinesscentral.model.sales_credit_memo_line import SalesCreditMemoLine
from pybusinesscentral.model.sales_order import SalesOrder
from pybusinesscentral.model.sales_order_line import SalesOrderLine
from pybusinesscentral.model.sales_quote import SalesQuote
from pybusinesscentral.model.sales_quote_line import SalesQuoteLine
from pybusinesscentral.model.shipment_method import ShipmentMethod
from pybusinesscentral.model.tax_area import TaxArea
from pybusinesscentral.model.tax_group import TaxGroup
from pybusinesscentral.model.time_registration_entry import TimeRegistrationEntry
from pybusinesscentral.model.trial_balance import TrialBalance
from pybusinesscentral.model.unit_of_measure import UnitOfMeasure
from pybusinesscentral.model.unitofmeasuretype import Unitofmeasuretype
from pybusinesscentral.model.vendor import Vendor
from pybusinesscentral.model.vendor_purchase import VendorPurchase

__all__ = [
    "Account",
    "AgedAccountsPayable",
    "AgedAccountsReceivable",
    "Attachments",
    "BalanceSheet",
    "BankAccount",
    "CashFlowStatement",
    "Company",
    "CompanyInformation",
    "CountryRegion",
    "Currency",
    "Customer",
    "CustomerFinancialDetail",
    "CustomerPayment",
    "CustomerPaymentJournal",
    "CustomerSale",
    "DefaultDimensions",
    "Dimension",
    "DimensionLine",
    "DimensionSetLine",
    "DimensionValue",
    "Dimensiontype",
    "Documentlineobjectdetailstype",
    "Employee",
    "GeneralLedgerEntry",
    "GeneralLedgerEntryAttachments",
    "IncomeStatement",
    "Item",
    "ItemCategory",
    "Itemunitofmeasureconversiontype",
    "Journal",
    "JournalLine",
    "PaymentMethod",
    "PaymentTerm",
    "PdfDocument",
    "Picture",
    "PostCustomerRequest",
    "PostDimensionSetLineRequest",
    "PostPurchaseInvoiceRequest",
    "PostPurchaseInvoiceLineForPurchaseInvoiceRequest",
    "PostSalesCreditMemoRequest",
    "PostSalesCreditMemoLineForSalesCreditMemoRequest",
    "PostSalesInvoiceRequest",
    "PostSalesInvoiceLineForSalesInvoiceRequest",
    "PostVendorRequest",
    "Postaladdresstype",
    "Project",
    "PurchaseInvoice",
    "PurchaseInvoiceLine",
    "RetainedEarningsStatement",
    "SalesCreditMemo",
    "SalesCreditMemoLine",
    "SalesInvoice",
    "SalesInvoiceLine",
    "SalesOrder",
    "SalesOrderLine",
    "SalesQuote",
    "SalesQuoteLine",
    "ShipmentMethod",
    "TaxArea",
    "TaxGroup",
    "TimeRegistrationEntry",
    "TrialBalance",
    "UnitOfMeasure",
    "Unitofmeasuretype",
    "Vendor",
    "VendorPurchase"
]


# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.aged_accounts_payable import AgedAccountsPayable

class TestAgedAccountsPayable(unittest.TestCase):
    """AgedAccountsPayable unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> AgedAccountsPayable:
        """Test AgedAccountsPayable
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `AgedAccountsPayable`
        """
        model = AgedAccountsPayable()
        if include_optional:
            return AgedAccountsPayable(
                vendor_id = '',
                vendor_number = '',
                name = '',
                currency_code = '',
                balance_due = 1.337,
                current_amount = 1.337,
                period1_amount = 1.337,
                period2_amount = 1.337,
                period3_amount = 1.337,
                aged_as_of_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                period_length_filter = ''
            )
        else:
            return AgedAccountsPayable(
        )
        """

    def testAgedAccountsPayable(self):
        """Test AgedAccountsPayable"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    (v2.0) Dynamics 365 Business Central

    (v2.0) Business Central Standard APIs

    The version of the OpenAPI document: 2.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from pybusinesscentral.model.employee import Employee

class TestEmployee(unittest.TestCase):
    """Employee unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Employee:
        """Test Employee
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Employee`
        """
        model = Employee()
        if include_optional:
            return Employee(
                id = '',
                number = '',
                display_name = '',
                given_name = '',
                middle_name = '',
                surname = '',
                job_title = '',
                address = pybusinesscentral.model.postaladdresstype.postaladdresstype(
                    street = '', 
                    city = '', 
                    state = '', 
                    country_letter_code = '', 
                    postal_code = '', 
                    customer_financial_details = [
                        pybusinesscentral.model.customer_financial_detail.customerFinancialDetail(
                            id = '', 
                            number = '', 
                            balance = 1.337, 
                            total_sales_excluding_tax = 1.337, 
                            overdue_amount = 1.337, )
                        ], 
                    picture = [
                        pybusinesscentral.model.picture.picture(
                            id = '', 
                            width = 56, 
                            height = 56, 
                            content_type = '', 
                            content@odata/media_edit_link = '', 
                            content@odata/media_read_link = '', )
                        ], 
                    default_dimensions = [
                        pybusinesscentral.model.default_dimensions.defaultDimensions(
                            parent_id = '', 
                            dimension_id = '', 
                            dimension_code = '', 
                            dimension_value_id = '', 
                            dimension_value_code = '', 
                            posting_validation = '', 
                            account = pybusinesscentral.model.account.account(
                                id = '', 
                                number = '', 
                                display_name = '', 
                                category = '', 
                                sub_category = '', 
                                blocked = True, 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                            dimension = pybusinesscentral.model.dimension.dimension(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                dimension_values = [
                                    pybusinesscentral.model.dimension_value.dimensionValue(
                                        id = '', 
                                        code = '', 
                                        display_name = '', 
                                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                    ], ), 
                            dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                        ], 
                    currency = pybusinesscentral.model.currency.currency(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        symbol = '', 
                        amount_decimal_places = '', 
                        amount_rounding_precision = 1.337, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_term = pybusinesscentral.model.payment_term.paymentTerm(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        due_date_calculation = '', 
                        discount_date_calculation = '', 
                        discount_percent = 1.337, 
                        calculate_discount_on_credit_memos = True, 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    shipment_method = pybusinesscentral.model.shipment_method.shipmentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                    payment_method = pybusinesscentral.model.payment_method.paymentMethod(
                        id = '', 
                        code = '', 
                        display_name = '', 
                        last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ),
                phone_number = '',
                mobile_phone = '',
                email = '',
                personal_email = '',
                employment_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                termination_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                status = '',
                birth_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                statistics_group_code = '',
                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                picture = [
                    pybusinesscentral.model.picture.picture(
                        id = '', 
                        width = 56, 
                        height = 56, 
                        content_type = '', 
                        content@odata/media_edit_link = '', 
                        content@odata/media_read_link = '', )
                    ],
                default_dimensions = [
                    pybusinesscentral.model.default_dimensions.defaultDimensions(
                        parent_id = '', 
                        dimension_id = '', 
                        dimension_code = '', 
                        dimension_value_id = '', 
                        dimension_value_code = '', 
                        posting_validation = '', 
                        account = pybusinesscentral.model.account.account(
                            id = '', 
                            number = '', 
                            display_name = '', 
                            category = '', 
                            sub_category = '', 
                            blocked = True, 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                        dimension = pybusinesscentral.model.dimension.dimension(
                            id = '', 
                            code = '', 
                            display_name = '', 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                            dimension_values = [
                                pybusinesscentral.model.dimension_value.dimensionValue(
                                    id = '', 
                                    code = '', 
                                    display_name = '', 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                ], ), 
                        dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                            id = '', 
                            code = '', 
                            display_name = '', 
                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                    ],
                time_registration_entries = [
                    pybusinesscentral.model.time_registration_entry.timeRegistrationEntry(
                        id = '', 
                        employee_id = '', 
                        employee_number = '', 
                        job_id = '', 
                        job_number = '', 
                        absence = '', 
                        line_number = 56, 
                        date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        quantity = 1.337, 
                        status = '', 
                        unit_of_measure_id = '', 
                        unit_of_measure = pybusinesscentral.model.unitofmeasuretype.unitofmeasuretype(
                            code = '', 
                            display_name = '', 
                            symbol = '', 
                            unit_conversion = pybusinesscentral.model.itemunitofmeasureconversiontype.itemunitofmeasureconversiontype(
                                to_unit_of_measure = '', 
                                from_to_conversion_rate = 1.337, 
                                picture = [
                                    pybusinesscentral.model.picture.picture(
                                        id = '', 
                                        width = 56, 
                                        height = 56, 
                                        content_type = '', 
                                        content@odata/media_edit_link = '', 
                                        content@odata/media_read_link = '', )
                                    ], 
                                default_dimensions = [
                                    pybusinesscentral.model.default_dimensions.defaultDimensions(
                                        parent_id = '', 
                                        dimension_id = '', 
                                        dimension_code = '', 
                                        dimension_value_id = '', 
                                        dimension_value_code = '', 
                                        posting_validation = '', 
                                        account = pybusinesscentral.model.account.account(
                                            id = '', 
                                            number = '', 
                                            display_name = '', 
                                            category = '', 
                                            sub_category = '', 
                                            blocked = True, 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), 
                                        dimension = pybusinesscentral.model.dimension.dimension(
                                            id = '', 
                                            code = '', 
                                            display_name = '', 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                            dimension_values = [
                                                pybusinesscentral.model.dimension_value.dimensionValue(
                                                    id = '', 
                                                    code = '', 
                                                    display_name = '', 
                                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                                                ], ), 
                                        dimension_value = pybusinesscentral.model.dimension_value.dimensionValue(
                                            id = '', 
                                            code = '', 
                                            display_name = '', 
                                            last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), )
                                    ], 
                                item_category = pybusinesscentral.model.item_category.itemCategory(
                                    id = '', 
                                    code = '', 
                                    display_name = '', 
                                    last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                            picture = [
                                pybusinesscentral.model.picture.picture(
                                    id = '', 
                                    width = 56, 
                                    height = 56, 
                                    content_type = '', 
                                    content@odata/media_edit_link = '', 
                                    content@odata/media_read_link = '', )
                                ], 
                            default_dimensions = [
                                pybusinesscentral.model.default_dimensions.defaultDimensions(
                                    parent_id = '', 
                                    dimension_id = '', 
                                    dimension_code = '', 
                                    dimension_value_id = '', 
                                    dimension_value_code = '', 
                                    posting_validation = '', )
                                ], 
                            item_category = pybusinesscentral.model.item_category.itemCategory(
                                id = '', 
                                code = '', 
                                display_name = '', 
                                last_modified_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), ), ), 
                        dimensions = [
                            null
                            ], 
                        last_modfied_date_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        project = pybusinesscentral.model.project.project(
                            id = '', 
                            number = '', 
                            display_name = '', ), )
                    ]
            )
        else:
            return Employee(
        )
        """

    def testEmployee(self):
        """Test Employee"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
